@@ -1,5 +1,5 @@
 -- Copyright (c) 2008--2011 Andres Loeh
--- Copyright (c) 2010--2018 Mikolaj Konarski and others (see git history)
+-- Copyright (c) 2010--2019 Mikolaj Konarski and others (see git history)
 -- This file is a part of the computer game Allure of the Stars
 -- and is released under the terms of the GNU Affero General Public License.
 -- For license and copyright information, see the file LICENSE.
@@ -10,17 +10,16 @@ module Content.ModeKindPlayer
   , playerMonster, playerAntiMonster, playerAnimal
   , playerHorror, playerMonsterTourist, playerHunamConvict
   , playerAnimalMagnificent, playerAnimalExquisite
-  , hiHero, hiDweller, hiRaid, hiEscapist
+  , hiHeroShort, hiHeroMedium, hiHeroLong, hiDweller
   , playerRobot
   ) where
 
 import Prelude ()
 
-import Game.LambdaHack.Common.Prelude
+import Game.LambdaHack.Core.Prelude
 
-import Game.LambdaHack.Common.Ability
-import Game.LambdaHack.Common.Misc
 import Game.LambdaHack.Content.ModeKind
+import Game.LambdaHack.Definition.Ability
 
 playerHero, playerAntiHero, playerCivilian, playerMonster, playerAntiMonster, playerAnimal, playerHorror, playerMonsterTourist, playerHunamConvict, playerAnimalMagnificent, playerAnimalExquisite :: Player
 playerRobot :: Player
@@ -31,7 +30,7 @@ playerHero = Player
   , fskillsOther = meleeAdjacent
   , fcanEscape = True
   , fneverEmpty = True
-  , fhiCondPoly = hiHero
+  , fhiCondPoly = hiHeroLong
   , fhasGender = True
   , ftactic = TExplore
   , fleaderMode = LeaderUI $ AutoLeader False False
@@ -49,7 +48,7 @@ playerCivilian = Player
   , fskillsOther = zeroSkills  -- not coordinated by any leadership
   , fcanEscape = False
   , fneverEmpty = True
-  , fhiCondPoly = hiDweller
+  , fhiCondPoly = hiHeroMedium
   , fhasGender = True
   , ftactic = TPatrol
   , fleaderMode = LeaderNull  -- unorganized
@@ -58,7 +57,7 @@ playerCivilian = Player
 
 playerMonster = Player
   { fname = "Alien Hierarchy"
-  , fgroups = ["monster", "mobile monster"]
+  , fgroups = ["monster", "mobile monster", "aquatic monster"]
   , fskillsOther = zeroSkills
   , fcanEscape = False
   , fneverEmpty = False
@@ -73,13 +72,14 @@ playerMonster = Player
   }
 
 playerAntiMonster = playerMonster
-  { fhasUI = True
-  , fleaderMode = LeaderUI $ AutoLeader True True
+  { fleaderMode = LeaderUI $ AutoLeader True True
+  , fhasUI = True
   }
 
 playerAnimal = Player
   { fname = "Animal Kingdom"
-  , fgroups = ["animal", "mobile animal", "immobile animal", "scavenger"]
+  , fgroups = [ "animal", "mobile animal", "immobile animal", "aquatic animal"
+              , "scavenger" ]
   , fskillsOther = zeroSkills
   , fcanEscape = False
   , fneverEmpty = False
@@ -97,7 +97,7 @@ playerAnimal = Player
 -- should be present or a horror player should be added to host them.
 playerHorror = Player
   { fname = "Horror Den"
-  , fgroups = [nameOfHorrorFact]
+  , fgroups = [horrorGroup]
   , fskillsOther = zeroSkills
   , fcanEscape = False
   , fneverEmpty = False
@@ -112,7 +112,7 @@ playerMonsterTourist =
   playerAntiMonster { fname = "Alien Tourist Office"
                     , fcanEscape = True
                     , fneverEmpty = True  -- no spawning
-                    , fhiCondPoly = hiEscapist
+                    , fhiCondPoly = hiHeroMedium
                     , ftactic = TFollow  -- follow-the-guide, as tourists do
                     , fleaderMode = LeaderUI $ AutoLeader False False }
 
@@ -122,9 +122,7 @@ playerHunamConvict =
 
 playerAnimalMagnificent =
   playerAnimal { fname = "Animal Magnificent Specimen Variety"
-               , fneverEmpty = True
-               , fleaderMode = -- False to move away from stairs
-                               LeaderAI $ AutoLeader True False }
+               , fneverEmpty = True }
 
 playerAnimalExquisite =
   playerAnimal { fname = "Animal Exquisite Herds and Packs Galore"
@@ -133,42 +131,64 @@ playerAnimalExquisite =
 victoryOutcomes :: [Outcome]
 victoryOutcomes = [Conquer, Escape]
 
-hiHero, hiRaid, hiDweller, hiEscapist :: HiCondPoly
+hiHeroLong, hiHeroMedium, hiHeroShort, hiDweller :: HiCondPoly
 
--- Heroes rejoice in loot.
-hiHero = [ ( [(HiLoot, 1000)]  -- multiplied by fraction of collected
-           , [minBound..maxBound] )
-         , ( [(HiConst, 1000), (HiLoss, -1)]
-           , victoryOutcomes )
-         ]
+hiHeroShort =
+  [ ( [(HiLoot, 100)]
+    , [minBound..maxBound] )
+  , ( [(HiConst, 100)]
+    , victoryOutcomes )
+  , ( [(HiSprint, -500)]  -- speed matters, but only if fast enough
+    , victoryOutcomes )
+  , ( [(HiSurvival, 10)]  -- few points for surviving long
+    , [minBound..maxBound] \\ victoryOutcomes )
+  ]
 
-hiRaid = [ ( [(HiLoot, 100)]
-           , [minBound..maxBound] )
-         , ( [(HiConst, 100)]
-           , victoryOutcomes )
-         ]
+hiHeroMedium =
+  [ ( [(HiLoot, 200)]  -- usually no loot, but if so, no harm
+    , [minBound..maxBound] )
+  , ( [(HiConst, 200), (HiLoss, -10)]
+    , victoryOutcomes )
+  , ( [(HiSprint, -500)]  -- speed matters, but only if fast enough
+    , victoryOutcomes )
+  , ( [(HiBlitz, -100)]  -- speed matters always
+    , victoryOutcomes )
+  , ( [(HiSurvival, 10)]  -- few points for surviving long
+    , [minBound..maxBound] \\ victoryOutcomes )
+  ]
 
--- Spawners or skirmishers get no points from loot, but try to kill
+-- Heroes in long crawls rejoice in loot.
+hiHeroLong =
+  [ ( [(HiLoot, 10000)]  -- multiplied by fraction of collected
+    , [minBound..maxBound] )
+  , ( [(HiSprint, -20000)]  -- speedrun bonus, if below this number of turns
+    , victoryOutcomes )
+  , ( [(HiBlitz, -100)]  -- speed matters always
+    , victoryOutcomes )
+  , ( [(HiSurvival, 10)]  -- few points for surviving long
+    , [minBound..maxBound] \\ victoryOutcomes )
+  ]
+
+-- Spawners get no points from loot, but try to kill
 -- all opponents fast or at least hold up for long.
 hiDweller = [ ( [(HiConst, 1000)]  -- no loot, so big win reward
               , victoryOutcomes )
             , ( [(HiConst, 1000), (HiLoss, -10)]
               , victoryOutcomes )
+            , ( [(HiSprint, -1000)]  -- speedrun bonus, if below
+              , victoryOutcomes )
             , ( [(HiBlitz, -100)]  -- speed matters
               , victoryOutcomes )
             , ( [(HiSurvival, 100)]
               , [minBound..maxBound] \\ victoryOutcomes )
             ]
 
-hiEscapist = ( [(HiLoot, 200)]  -- loot matters a little bit
-             , [minBound..maxBound] )
-             : hiDweller
-
 -- Allure-specific
 
 playerRobot = Player
   { fname = "Robot Anarchy"
-  , fgroups = ["robot", "mobile robot", "immobile robot", "construction robot"]
+  , fgroups = [ "robot", "mobile robot", "immobile robot" --, "aquatic robot"
+              , "construction robot" ]
   , fskillsOther = zeroSkills
   , fcanEscape = False
   , fneverEmpty = False
